package io.zipcoder.tc_spring_poll_application.exception;

import io.zipcoder.tc_spring_poll_application.dto.error.ErrorDetail;
import io.zipcoder.tc_spring_poll_application.dto.error.ValidationError;
import org.springframework.context.MessageSource;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@ControllerAdvice
public class RestExceptionHandler {

    private MessageSource messageSource;


    @ExceptionHandler(ResourceNotFoundError.class)
    ResponseEntity<?> ResourceNotFoundError(ResourceNotFoundError rnfe, HttpServletRequest request) throws Exception{

        ErrorDetail errors = new ErrorDetail();

        errors.setTimeStamp(new Date().getTime());
        errors.setStatus(HttpStatus.NOT_FOUND.value());
        errors.setTitle("Resource not Found");
        errors.setDetail(rnfe.getMessage());
        errors.setDeveloperMessage(rnfe.getClass().getName());

        return new ResponseEntity<>(errors,null,HttpStatus.NOT_FOUND);

    }
    @ExceptionHandler(MethodArgumentNotValidException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public @ResponseBody
    ErrorDetail handleValidationError(  MethodArgumentNotValidException manve, HttpServletRequest request){

        ErrorDetail errorDetail = new ErrorDetail();
        errorDetail.setTimeStamp(new Date().getTime());
        errorDetail.setStatus(HttpStatus.BAD_REQUEST.value());
        String requestPath = (String) request.getAttribute("java.servlet.error.request_uri");

        if(requestPath == null){
            requestPath = request.getRequestURI();
        }

        errorDetail.setTitle("Bad entry, Validating Error");
        errorDetail.setDetail("Input validation fail");
        errorDetail.setDeveloperMessage(manve.getClass().getName());

        List<FieldError> fieldErrors =  manve.getBindingResult().getFieldErrors();
        for(FieldError fe : fieldErrors) {

            List<ValidationError> validationErrorList = errorDetail.getErrors().get(fe.getField());
            if(validationErrorList == null) {
                validationErrorList = new ArrayList<ValidationError>();
                errorDetail.getErrors().put(fe.getField(), validationErrorList);
            }
            ValidationError validationError = new ValidationError();
            validationError.setCode(fe.getCode());
            validationError.setMessage(messageSource.getMessage(fe, null));
            validationErrorList.add(validationError);
        }
        return errorDetail;
    }




}
